@echo off
cls 
SetLocal EnableExtensions EnableDelayedExpansion

REM Check for administratior privileges. If not - please start as admin!
net session >nul 2>&1
IF NOT %ERRORLEVEL%==0 (
  ECHO You must right-click and select
  ECHO "RUN AS ADMINISTRATOR"  to run this batch. Exiting... 
  ECHO.
  PAUSE
  EXIT /D
)
  
echo.
echo  ###################################################################
echo  #                 Kontron - Product Center Boards                 #
echo  #                       BIOS Update script                        #
echo  ###################################################################

pushd "%~dp0"
set AFU=Afu\AFUWINx64.exe
set AFUParams=/B /P /N /K:A4FB2BAACD9847ca86A1C333225764DC /RLC:F
set MeDevId=7AE8
set FWUpdLcl=FWUpdLcl\FWUpdLcl64.exe
set FWUpdLclParams=-Y -F

set FlashFile=%~n1%~x1

REM Set variables which have to remain after reboots
if not exist  _statusvars.cmd (
  REM if file does not exist write header to it
  echo. > _statusvars.cmd
  echo REM This file is needed for the update process. Please do not remove manually. >> _statusvars.cmd
  echo @echo off >> _statusvars.cmd
) else (
  REM Execute file
  call _statusvars.cmd > nul
)

REM Read SMBIOS type 2 (Baseboard) to get the running board product type
FOR /F "skip=2 tokens=2 delims=," %%A IN ('wmic baseboard get product /FORMAT:csv') do (set "product=%%A")

REM Read SMBIOS type 0 (BIOS) to get the running BIOS version
FOR /F "skip=2 tokens=2 delims= " %%A IN ('wmic bios get SmBiosBiosVersion /FORMAT:csv') do (set "BIOSV=%%A")

REM If this script was called with an argument, assume it's the BIOS flash filename
REM Continue without searching for a suitable file
IF "%FlashFile%"=="" (
  REM Search for a suitable one in the script directory, with highest revision
  FOR %%F IN (%product%*.bin) do (
          if /i %%~nxF GTR !FlashFile! (
        set FlashFile=%%~nxF
          )
  )
  REM Check if suitable flash BIN file was found
  IF "!FlashFile!" == "" goto ERR_FILE
)

REM BIOS version from Filename 
call set BIOS_TARGET_VERSION=%%FlashFile:%product%.=%%
set BIOS_TARGET_VERSION=%BIOS_TARGET_VERSION:.BIN=%

REM Check BIOS Version
if %BIOSV% == %BIOS_TARGET_VERSION% (
  echo.
  echo  o BIOS for motherboard  : %product%
  echo  o BIOS version installed: %BIOSV%
  echo  o BIOS version of file  : %BIOS_TARGET_VERSION%
  echo.
  echo  ###################################################################
  echo  #                                                                 #
  echo  #           The Target BIOS version is already installed.         #
  echo  #                Do you want to flash the BIOS again?             #
  echo  #                          ^<y^>es ^| ^<n^>o                           #
  echo  #                                                                 #
  echo  ###################################################################
  echo.
  choice /C yn /N > nul
  IF ERRORLEVEL 2 GOTO EOF
  IF ERRORLEVEL 1 cls && GOTO UPDATE_START
)

:UPDATE_START
REM Check if according ME flash BIN file exists
call set MeFlashFile=MEFW_%FlashFile%
if not exist %MeFlashFile% (
  goto ERR_ME_FILE
)

REM Determine status of ME Interface
wmic path win32_pnpentity where "deviceid like '%%DEV_%%%MeDevId%%%%%'" get status 2> nul | findstr /i "Error OK" > nul
if not %ERRORLEVEL% == 0 (
  REM ME is deactivated
    goto ERR_MEinactive
  )

REM Check ROM ID from flashfile and compare it with product name of the board
%AFU% %FlashFile% /U | findstr /I "%product%" > nul
if not %ERRORLEVEL% == 0 goto ERR_ROMID

REM Show message to user
:EXECUTE
echo.
echo  o BIOS for motherboard  : %product%
echo  o BIOS version installed: %BIOSV%
echo  o BIOS File             : %FlashFile%
echo  o MEFW File             : %MeFlashFile%
echo.
  echo  ###################################################################
  echo  #                                                                 #
echo  #         Are you sure to update the ME and BIOS Firmware?        #
  echo  #                          ^<y^>es ^| ^<n^>o                           #
  echo  #                                                                 #
  echo  #      ATTENTION: System is blocked during BIOS update!           #
  echo  #                 To prevent data loss it's recommended to        #
  echo  #                 save and close all open documents.              #
  echo  #                                                                 #
  echo  ###################################################################
  echo.
  choice /C yn /N > nul
  IF ERRORLEVEL 2 GOTO EOF
IF ERRORLEVEL 1 GOTO FLASH_ME

:FLASH_ME
echo  ME Firmware update initiated. Please wait...
REM Check if ME Interface is installed
wmic path win32_pnpentity where "name like '%%Management Engine Interface%%'" get present 2> nul | findstr /i "true" > nul
if %ERRORLEVEL% == 0 (
  %FWUpdLcl% %FWUpdLclParams% %MeFlashFile%
  if not !ERRORLEVEL! == 0 (
    if !ERRORLEVEL! == 5130 (
      REM ME is not accessible
      goto ERR_MEinactive
    )
    REM Ignore errorlevel 84748 (Firmware version is already installed)
    if !ERRORLEVEL! == 84748 (
      cls
      echo.
      echo. ME firmware is already up to date. Continue with BIOS firmware update...
      echo.
      goto FLASH_START
    )
    REM Ignore errorlevel 91429 (Update to Image with lower VCN not allowed)
    if !ERRORLEVEL! == 91429 (
      cls
      echo.
      echo. ME firmware flash is not allowed due to lower VCN in file image.
      echo. Continue with BIOS firmware update...
      echo.
      goto FLASH_START
    ) else (
      goto ERR_FWUpdLcl
    )

  )
) else (
  if "%ME_installed%" == "true" (
    goto ERR_FWUpdLcl
  )
  REM install driver
  pnputil /add-driver FWUpdLcl\heci.inf /install > nul
  if not !ERRORLEVEL! == 0 goto ERR_MEinactive
  echo set ME_installed=true>> _statusvars.cmd
  call _statusvars.cmd > nul
  goto FLASH_ME
)

:FLASH_START
set EC_HOWTO_SHOWN=
%AFU% %FlashFile% %AFUParams%
set AFUERR=%ERRORLEVEL%
if NOT %AFUERR%==0 (
  goto ERR_AFU
)

:FLASH_DONE
del /f _statusvars.cmd > nul

REM Automatic restart...
cls && color A
echo.
echo  ###################################################################
echo  #  DONE: Flash successfull. System has to be restarted^^!            #
echo  #                                                                 #
echo  #  HINT: In general a power cycle is suggested...                 #
echo  #          1. Wait until the system switched off                  #
echo  #          2. Remove AC cable                                     #
echo  #          3. Wait until all voltage rails are discharged         #
echo  #          4. Re-connect AC cable an switch system on             #
echo  #                                                                 #
if "%ME_installed%" == "true" (
  echo  #  INFO: The driver for "Management Engine Interface" was         #
  echo  #        installed automatically. In case you want a "clean"      #
  echo  #        system, you will have to remove it manually.             #
  echo  #                                                                 #
)
echo  #  Do you want to shutdown now? ^<y^>es ^| ^<n^>o                      #
echo  #                                                                 #
echo  ###################################################################
echo.
choice /C yn /N > nul
IF ERRORLEVEL 2 GOTO EOF
IF ERRORLEVEL 1 shutdown -s -f -t 0

:ERR_FILE
echo. 
echo  ###################################################################
echo  #   ERROR: Flash file and board does not match^^!                   #
echo  ###################################################################
echo. 
goto EOF

:ERR_ME_FILE
echo.
echo  ###################################################################
echo  #   ERROR: ME Flash file not found!                               #
echo  ###################################################################
echo.
goto EOF      

:ERR_ROMID
echo.
echo  ###################################################################
echo  #   ERROR: ROM ID from Flashfile does not match to system^^!        #
echo  ###################################################################
echo.
goto EOF 

:ERR_AFU
echo. 
echo  ###################################################################
echo  #   ERROR: AFU flash tool reported an error.                      #
echo  #          Please check and try again                             #
echo  ###################################################################
echo. 
echo  Returncode: %AFUERR%
echo.
goto EOF

:ERR_FWUpdLcl
echo. 
echo  ###################################################################
echo  #   ERROR: FWUpdLcl reported an error.                            #
echo  #          Please check and try again                             #
echo  ###################################################################
echo. 
echo  Returncode: %FWUpdLclERR%
echo.
goto EOF   

:ERR_MEinactive
echo. 
echo  ###################################################################
echo  #   ERROR: ME is not running.                                     #
echo  #          Please reboot and re-run the script afterwards.        #
echo  #                                                                 #
echo  #          Do you want to reboot now? ^<y^>es ^| ^<n^>o                #
echo  #                                                                 #
echo  ###################################################################
echo.
choice /C yn /N > nul
IF ERRORLEVEL 2 GOTO EOF
IF ERRORLEVEL 1 shutdown -r -f -t 0
goto EOF


:EOF
pause
color 7
